-- Global Variables
HealVsDamage_UNIT_COMBAT = "UNIT_COMBAT"
HealVsDamage_UNIT_POWER = "UNIT_POWER"
HealVsDamage_TARGET_CHANGED = "PLAYER_TARGET_CHANGED"
HealVsDamage_values = HealVsDamage_Data:new()
HealVsDamage_debug = false

-- Functions

function HealVsDamage_OnLoadEvent(frame)
	frame:RegisterEvent(HealVsDamage_UNIT_COMBAT)
	frame:RegisterEvent(HealVsDamage_UNIT_POWER)
	frame:RegisterEvent(HealVsDamage_TARGET_CHANGED)
	print("Heal Vs. Damage Healer AddOn 1.2.0 Loaded")
end

function HealVsDamage_EventHandler(self, event, ...)
	local args = {...}
	
	if event == HealVsDamage_UNIT_COMBAT then
		HealVsDamage_CombatEvent(args[1], args[2], args[4])
	elseif event == HealVsDamage_TARGET_CHANGED then
		HealVsDamage_TargetChanged()
	elseif event == HealVsDamage_UNIT_POWER then
		HealVsDamage_UnitPower(args[1], args[2]);
	end
end

function HealVsDamage_UnitPower(unit, resource)
	isPlayer = unit == "player"
	if not isPlayer then
		return
	end
	
	isMana = resource == "MANA"
	if not isMana then
		return
	end
	
	HealVsDamage_values:updateMana(UnitPower("player", SPELL_POWER_MANA))
	HealVsDamage_UpdateFrames()
end

function HealVsDamage_TargetChanged()
	shouldHide = UnitHealth("target") == 0

	if shouldHide then
		HealVsDamageFrame:Hide()
		HealVsDamageFrameMana:Hide()
	else
		HealVsDamage_values = HealVsDamage_Data:new()
		
		if UnitCanAttack("player", "target") then
			HealVsDamage_values.targetId = "player"
		else
			HealVsDamage_values.targetId = "target"
		end
		
		if HealVsDamage_debug then
			print("targetId Changed to " .. HealVsDamage_values.targetId)
		end
		
		HealVsDamageFrame:Show()
		HealVsDamageFrameMana:Show()
		HealVsDamage_UpdateFrames()
	end
end

function HealVsDamage_CombatEvent(unit, action, value)
	isTarget = unit == HealVsDamage_values.targetId

	if HealVsDamage_debug then
		isTargetString = "false"
	
		if isTarget then
			isTargetString = "true"
		end
		
		print("Unit: " .. unit .. ", Action: " .. action .. ", Value: " .. value .. ", isTarget: " .. isTargetString)
	end
	if not isTarget then
		return
	end
	
	if HealVsDamage_debug then
		print("Event will be registered")
	end
	
	if action == "HEAL" then
		HealVsDamage_values:addHealing(value)
	elseif action == "WOUND" then
		HealVsDamage_values:addDamage(value)
	end
	
	HealVsDamage_UpdateFrames();
end

function HealVsDamage_UpdateFrames_Mana()
	used = HealVsDamage_values:getManaUsage()
	
	base, casting = GetManaRegen()
	expectedLimit = casting * 10
	
	if used == 0 then
		HealVsDamage_Mana_Gain:Hide()
		HealVsDamage_Mana_Loss:Hide()
	elseif used > 0 then
		usedPercentage = HealVsDamageFrameMana:GetWidth() * math.min(1, (used / expectedLimit))
		HealVsDamage_Mana_Gain:Hide()
		HealVsDamage_Mana_Loss:Show()
		HealVsDamage_Mana_Loss:SetWidth(usedPercentage)
		HealVsDamage_ManaDiffString:SetText(-used)
	else
		usedPercentage = HealVsDamageFrameMana:GetWidth() * math.min(1, (math.abs(used) / expectedLimit))
		HealVsDamage_Mana_Gain:Show()
		HealVsDamage_Mana_Loss:Hide()
		HealVsDamage_Mana_Gain:SetWidth(usedPercentage)
		HealVsDamage_ManaDiffString:SetText(math.abs(used))
	end
end

function HealVsDamage_UpdateFrames_HealDamage()
	sum = HealVsDamage_values:getDamageAndHealingSum()
	if sum == 0 then
		HealVsDamage_Damage:SetWidth(HealVsDamageFrame:GetWidth() / 2)
		HealVsDamage_Heal:SetWidth(HealVsDamageFrame:GetWidth() / 2)
		HealVsDamage_DiffString:SetText(0)
		return
	end
	
	val = HealVsDamage_values
	
	healing = val:getHealing();
	damage = val:getDamage();
	
	healPercentage = healing * HealVsDamageFrame:GetWidth() / sum
	dmgPercentage = damage * HealVsDamageFrame:GetWidth() / sum
	diff = healing - damage
	
	HealVsDamage_Damage:SetWidth(dmgPercentage)
	HealVsDamage_Heal:SetWidth(healPercentage)
	
	HealVsDamage_DiffString:SetText(diff)
end

function HealVsDamage_UpdateFrames()
	HealVsDamage_UpdateFrames_HealDamage();
	HealVsDamage_UpdateFrames_Mana();
end
